unit LocalCDDB;
{$DEFINE INIT_AT_CREATE} //Define to make the component initialize on creation
interface
uses Windows,
  SysUtils,
  Cddb,
  Dialogs,
  AspiCD;

type

  ENoDLL = Exception;

  TCppStringArray = array[0..99] of PChar;

  TCppCDInfo = record
    DiscID : PChar;
    Artist : PChar;
    Title : PChar;
    Year : PChar;
    Genre : PChar;
    AddAlbumInfo : PChar;
    Tracks : TCppStringArray;
    AddTrkInfo : TCppStringArray;
    Category : PChar;
    Revision : Integer;
    CDToc : TCDToc;
  end;

  PInfo = ^TCppCDInfo;
  TWrite = function( info : PInfo; path : PChar; os : Bool ) : integer;
  stdcall;
  TVersion = procedure( version : PChar; len : integer ); stdcall;
  TRead = function( info : PInfo; path : PChar; discid : PChar; os : Bool ) :
    integer; stdcall;
  TCreateCDDBString = function( const info : PInfo; szReturn : PChar;
    nBufferLength : integer ) : integer; stdcall;
  TParseCDDBString = function( info : PInfo; szString : PChar ) : integer;
  stdcall;

  TMatchArray = array[0..99] of PChar;

  TFindMatches = function( info : PCDInfo; path : PChar; os : BOOL; aMatches :
    TMatchArray ) : integer; stdcall;

  TLocalCDDB = class( TObject )
  private
    hDLL : THandle;
    _WriteCDData : TWrite;
    _SetVersion : TVersion;
    _ReadCDData : TRead;
    _CreateCDDBString : TCreateCDDBString;
    _ParseCDDBString : TParseCDDBString;
    _FindMatches : TFindMatches;
    function ConvertInfoPtoCpp( Info : TCDInfo ) : TCppCDInfo;
    function ConvertInfoCpptoP( Info : TCppCDInfo ) : TCDInfo;
  protected
    //  FInitialized : Boolean;
    FCDDBPath : string;
    FDBType : Bool;
  public
    constructor Create( DLLPath : string );
    destructor Destroy; override;
    procedure Free;
    function WriteCDData( Info : TCDInfo ) : integer;
    procedure SetVersion( Version : string; LenOfLine : integer = 512 );
    function ReadCDData( var Info : TCDInfo ) : integer;
    function ParseCDDBString( var Info : TCDInfo; Entry : TEntryArray ) :
      integer;
    function CreateCDDBString( Info : TCDInfo; var Entry : TEntryArray ) :
      integer;
    function FindMatches( Info : TCDInfo; var Matches : TMatches ) : integer;
  published
    //  property Initialized : Boolean read FInitialized;
    property CDDBPath : string read FCDDBPath write FCDDBPath;
    property DBType : Bool read FDBType write FDBType default False;
    //Below: False = UNIX
  end;

const
  LineLength = 512;

  NotInitialized = -5; //Just in case the lowest one
  WIN = True;
  UNIX = False;
  {/// WriteCDDatan virheet }
  WD_ERR_ARG = -1; {// Virheelliset argumentit}
  WD_ERR_DIR = -2; {// Virhe luotaessa hakemistoa}
  WD_ERR_FILE_CREATE = -3; {// Virhe luotaessa tiedostoa}
  WD_ERR_FILE_WRITE = -4; {// Virhe kirjoittaessa tiedostoon}

  {/// ReadCDDatan virheet }
  RD_ERR_ARG = -1; {// Virheelliset argumentit}
  RD_ERR_FILE = -2; {// Tiedostoa ei lytynyt}
  RD_ERR_READ = -3; {// Lukuvirhe}
  RD_ERR_CORRUPT = -4; {// Tiedosto hajalla}

  {/// CreateCDDBStringin virheet }
  CS_ERR_OVERFLOW = -1; {// Liian pieni bufferi}

  {/// FindMatchesin virheet }
  FM_ERR_MATCH = 0; // Matchi lytyi
  FM_ERR_NOMATCH = -1; // Matchia ei lytynyt

var
  //  EntryCharArray : array[0..511] of array[0..LineLength - 1] of Char;

  DiscIdCharArray : array[0..LineLength - 1] of Char;
  ArtistCharArray : array[0..LineLength - 1] of Char;
  TitleCharArray : array[0..LineLength - 1] of Char;
  YearCharArray : array[0..LineLength - 1] of Char;
  GenreCharArray : array[0..LineLength - 1] of Char;
  AddAlbumInfoCharArray : array[0..LineLength - 1] of Char;
  CategoryCharArray : array[0..LineLength - 1] of Char;

  TracksCharArray : array[0..99] of array[0..LineLength - 1] of Char;
  AddTrkInfoCharArray : array[0..99] of array[0..LineLength - 1] of Char;

  CppInfo : TCppCDInfo;

implementation

procedure EntryStrToEntryArray( var EntryArray : TEntryArray; EntryStr : PChar
  );
var
  p, q, r : PChar;
  i : integer;
begin
  p := EntryStr;
  q := p + Length( EntryStr ) - 1;
  r := p;
  i := -1;
  while ( p <> nil ) and ( p <= q ) do
  begin
    while ( p <= q ) and ( p^ <> #13 ) and ( p^ <> #10 ) do
      Inc( p );
    Inc( i );
    EntryArray[i] := ( Copy( StrPas( r ), 1, p - r ) );
    if ( p[0] = #13 ) and ( p[1] = #10 ) then
      Inc( p, 2 )
    else
      Inc( p );
    r := p;
  end;
end;

procedure ResetCppInfo;
var
  i : integer;

  procedure ZeroMemoryEx( Dest : Pointer );
  begin
    ZeroMemory( Dest, LineLength - 1 );
  end;

begin
  with CppInfo do
  begin
    //StrPCopy( DiscIDCharArray, '' );
    ZeroMemoryEx( @DiscIdCharArray );
    DiscID := @DiscIdCharArray;
    //StrPCopy( ArtistCharArray, '' );
    ZeroMemoryEx( @ArtistCharArray );
    Artist := @ArtistCharArray;
    //StrPCopy( TitleCharArray, '' );
    ZeroMemoryEx( @TitleCharArray );
    Title := @TitleCharArray;
    //StrPCopy( YearCharArray, '' );
    ZeroMemoryEx( @YearCharArray );
    Year := @YearCharArray;
    //StrPCopy( GenreCharArray, '' );
    ZeroMemoryEx( @GenreCharArray );
    Genre := @GenreCharArray;
    //StrPCopy( AddAlbumInfoCharArray, '' );
    ZeroMemoryEx( @AddAlbumInfoCharArray );
    AddAlbumInfo := @AddAlbumInfoCharArray;
    //StrPCopy( CategoryCharArray, '' );
    ZeroMemoryEx( @CategoryCharArray );
    Category := @CategoryCharArray;

    for i := 0 to 99 do
    begin
      //StrPCopy( TracksCharArray[i], '' );
      ZeroMemoryEx( @TracksCharArray[i] );
      Tracks[i] := @TracksCharArray[i];
      //StrPCopy( AddTrkInfoCharArray[i], '' );
      ZeroMemoryEx( @AddTrkInfoCharArray[i] );
      AddTrkInfo[i] := @AddTrkInfoCharArray[i];
    end;

    Revision := 0;

  end;
end;

procedure AllocCppInfo( var Info : TCppCDInfo );
var
  i : integer;
begin
  with Info do
  begin
    DiscID := StrAlloc( LineLength );
    Artist := StrAlloc( LineLength );
    Title := StrAlloc( LineLength );
    Year := StrAlloc( LineLength );
    Genre := StrAlloc( LineLength );
    AddAlbumInfo := StrAlloc( LineLength );
    Category := StrAlloc( LineLength );

    for i := 0 to 99 do
    begin
      AddTrkInfo[i] := StrAlloc( LineLength );
      Tracks[i] := StrAlloc( LineLength );
    end;

  end;
end;

procedure DisposeCppInfo( var Info : TCppCDInfo );
var
  i : integer;
begin
  with Info do
  begin
    StrDispose( DiscID );
    StrDispose( Artist );
    StrDispose( Title );
    StrDispose( Year );
    StrDispose( Genre );
    StrDispose( AddAlbumInfo );
    StrDispose( Category );

    for i := 0 to 99 do
    begin
      StrDispose( AddTrkInfo[i] );
      StrDispose( Tracks[i] );
    end;
  end;
end;

constructor TLocalCDDB.Create( DLLPath : string );
begin
  inherited Create;
  //  FInitialized := False;
  FCDDBPath := ExtractFilePath( DLLPath );     // The initial default
  hDLL := LoadLibrary( PChar( DLLPath ) );
  if hDLL <> 0 then
  begin
    @_SetVersion := GetProcAddress( hDLL, 'SetVersion' );
    @_WriteCDData := GetProcAddress( hDLL, 'WriteCDData' );
    @_ParseCDDBString := GetProcAddress( hDLL, 'ParseCDDBString' );
    @_ReadCDData := GetProcAddress( hDLL, 'ReadCDData' );
    @_CreateCDDBString := GetProcAddress( hDLL, 'CreateCDDBString' );
    @_FindMatches := GetProcAddress( hDLL, 'FindMatches' );
    if ( @_SetVersion = nil ) or ( @_WriteCDData = nil ) or
      ( @_ParseCDDBString = nil ) or ( @_ReadCDData = nil ) or (
      @_CreateCDDBString = nil ) or ( @_FindMatches = nil ) then
    begin
      raise ENoDll.Create( 'Dll is invalid. Maybe a wrong version?' );
      Exit;
    end;
  end
  else
  begin
    raise ENoDll.Create( 'Dll not found' );
    Exit;
  end;

  ResetCppInfo;

{$IFDEF INIT_AT_CREATE}
  SetVersion( 'Ari''s freedb component v0.1.70 Release', LineLength );
{$ENDIF}
end;

destructor TLocalCDDB.Destroy;
begin
  _WriteCDData := nil;
  _SetVersion := nil;
  _ReadCDData := nil;
  _CreateCDDBString := nil;
  _ParseCDDBString := nil;
  FreeLibrary( hDLL );
  inherited Destroy;
end;

procedure TLocalCDDB.Free;
begin
  if Self <> nil then
    Destroy;
end;

function TLocalCDDB.ConvertInfoPtoCpp( Info : TCDInfo ) : TCppCDInfo;
var
  i : integer;
begin
  // Converts a "Pascal type" info into a "C++ type" info
  with Result do
  begin
    StrPCopy( DiscIDCharArray, Info.DiscID );
    DiscID := @DiscIDCharArray;
    //    DiscID := PChar( Info.DiscID );
    StrPCopy( ArtistCharArray, Info.Artist );
    Artist := @ArtistCharArray;
    //    Artist := PChar( Info.Artist );
    StrPCopy( TitleCharArray, Info.Title );
    Title := @TitleCharArray;
    //    Title := PChar( Info.Title );
    StrPCopy( YearCharArray, Info.Year );
    Year := @YearCharArray;
    //    Year := PChar( Info.Year );
    StrPCopy( GenreCharArray, Info.Genre );
    Genre := @GenreCharArray;
    //    Genre := PChar( Info.Genre );
    StrPCopy( AddAlbumInfoCharArray, Info.AddAlbumInfo );
    AddAlbumInfo := @AddAlbumInfoCharArray;
    //    AddAlbumInfo := PChar( Info.AddAlbumInfo );
    StrPCopy( CategoryCharArray, Info.Category );
    Category := @CategoryCharArray;
    //    Category := PChar( Info.Category );
    Revision := Info.Revision;
    CDToc := Info.CDToc;

    for i := 0 to 99 do
    begin
      StrPCopy( TracksCharArray[i], Info.Tracks[i] );
      Tracks[i] := @TracksCharArray[i];
      //      Tracks[i] := PChar( Info.Tracks[i] );
      StrPCopy( AddTrkInfoCharArray[i], Info.AddTrkInfo[i] );
      AddTrkInfo[i] := @AddTrkInfoCharArray[i];
      //      AddTrkInfo[i] := PChar( Info.AddTrkInfo[i] );
    end;
  end;

end;

function TLocalCDDB.ConvertInfoCpptoP( Info : TCppCDInfo ) : TCDInfo;
var
  i : integer;
begin
  // Converts a "C++ type" info into a "Pascal type" info
  with Result do
  begin
    DiscID := Info.DiscID;
    Artist := Info.Artist;
    Title := Info.Title;
    Year := Info.Year;
    Genre := Info.Genre;
    AddAlbumInfo := Info.AddAlbumInfo;
    Category := Info.Category;
    Revision := Info.Revision;
    CDToc := Info.CDToc;

    for i := 0 to 99 do
    begin
      Tracks[i] := Info.Tracks[i];
      AddTrkInfo[i] := Info.AddTrkInfo[i];
    end;
  end;

end;

function TLocalCDDB.WriteCDData( Info : TCDInfo ) : integer;

begin
  //  AllocCppInfo( CppInfo );
  ResetCppInfo;
  CppInfo := ConvertInfoPtoCpp( Info );
  Result := _WriteCDData( @CppInfo, PChar( FCDDBPath ), FDBType );
  ResetCppInfo;
  //  DisposeCppInfo( CppInfo );
end;

procedure TLocalCDDB.SetVersion( Version : string; LenOfLine : integer = 512 );
begin
  _SetVersion( PChar( Version ), LenOfLine );
end;

function TLocalCDDB.ReadCDData( var Info : TCDInfo ) : integer;

begin
  //  AllocCppInfo( CppInfo );
  ResetCppInfo;
  //  CppInfo := ConvertInfoPtoCpp( Info );
  Result := _ReadCDData( @CppInfo, PChar( FCDDBPath ), PChar( Info.DiscID ),
    FDBType
    );
  Info := ConvertInfoCpptoP( CppInfo );
  ResetCppInfo;
  //  DisposeCppInfo( CppInfo );
end;

function TLocalCDDB.ParseCDDBString( var Info : TCDInfo; Entry : TEntryArray ) :
  integer;
var
  i : integer;
  //EntryCharArray : array[0..16383] of Char; //just in case
  EntryString : PChar;
  Temp : string;
begin
  //  AllocCppInfo( CppInfo );
  ResetCppInfo;
  Temp := '';
  for i := 0 to 511 do
  begin
    if Entry[i] = '' then
    begin
      break;
    end;
    Temp := Temp + Entry[i] + #13#10;
  end;
  //StrPCopy( EntryCharArray, Temp );
  GetMem( EntryString, Length( Temp ) + 512 ); //Just in case the additional 512
  StrPCopy( EntryString, Temp );
  //EntryString := @EntryCharArray;
  Result := _ParseCDDBString( @CppInfo, EntryString );
  //  MessageBox( 0, CppInfo.DiscID, 'Componentti', 0 );
  Info := ConvertInfoCpptoP( CppInfo );
  ResetCppInfo;
  FreeMem( EntryString );
  //  DisposeCppInfo( CppInfo );
end;

function TLocalCDDB.CreateCDDBString( Info : TCDInfo; var Entry : TEntryArray )
  : integer;
var
  szEntry : PChar;
  //  i : integer;
begin
  //  AllocCppInfo( CppInfo );
  ResetCppInfo;
  CppInfo := ConvertInfoPtoCpp( Info );

  GetMem( szEntry, 4096 );
  //  CppInfo.CDToc.LeadOut := Info.CDToc.LeadOut;
  Result := _CreateCDDBString( @CppInfo, szEntry, 4096 );
  if Result = CS_ERR_OVERFLOW then
  begin
    FreeMem( szEntry );
    GetMem( szEntry, 8192 );
    Result := _CreateCDDBString( @CppInfo, szEntry, 8192 );
    if Result = CS_ERR_OVERFLOW then
    begin
      FreeMem( szEntry );
      GetMem( szEntry, 16384 );
      Result := _CreateCDDBString( @CppInfo, szEntry, 16384 );
    end;
  end;

  EntryStrToEntryArray( Entry, szEntry );
  FreeMem( szEntry );
  ResetCppInfo;
  //  DisposeCppInfo( CppInfo );
end;

function TLocalCDDB.FindMatches( Info : TCDInfo; var Matches : TMatches ) :
  integer;

  function FindSpace( Line : string ) : integer;
  var
    i : integer;
  begin
    for i := 1 to Length( Line ) do
    begin
      if Line[i] = ' ' then
      begin
        Result := i;
        exit;
      end;
    end;
    Result := Length( Line );
  end;

var
  aMatches : TMatchArray;
  i : integer;
begin
  for i := 0 to 99 do
  begin
    GetMem( aMatches[i], 32 );
    ZeroMemory( aMatches[i], 32 );
  end;

  CppInfo := ConvertInfoPtoCpp( Info );

  Result := _FindMatches( @CppInfo, PChar( FCDDBPath ), FDBType, aMatches );
  if Result = FM_ERR_MATCH then
  begin
    for i := 0 to 99 do
    begin
      if ( aMatches[i] = '' ) or ( aMatches[i][1] = #0 ) then
      begin
        //We obviously do not have any more matches
        break;
      end;
      Matches[i].Category := Copy( aMatches[i], 1, FindSpace( aMatches[i] ) - 1
        );
      Matches[i].DiscID := Copy( aMatches[i], FindSpace( aMatches[i] ) + 1,
        Length(
        aMatches[i] ) - FindSpace( aMatches[i] ) + 1 );
    end;
  end
  else
    if Result = FM_ERR_NOMATCH then
    begin
      for i := 0 to 99 do
      begin
        Matches[i].Category := '';
        Matches[i].DiscID := '';
      end;
    end;

  for i := 0 to 99 do
  begin
    FreeMem( aMatches[i] );
  end;
end;

end.

