unit hARBC;

interface


// ARBC = [A]ssignFile - [R]eset - [B]lockRead/Write - [C]loseFile

type
  TARBC = class(TObject)
  private
    F        : File of Byte;
    hSize    : integer;
    hBuffer  : pointer;
    hOpened  : boolean;
    hFileName: string;

    function    GetFileName: string;
    procedure   SetFileName( FN: string );
    function    GetFileSize: int64;
    function    GetIOResult: integer;
    function    GetFilePos: int64;

  public
    constructor Create( Buffer: pointer; BufferSize: integer );
    destructor  Destroy;

    property    IOResult: integer read GetIOResult;
    property    FileName: string read GetFileName write SetFileName;
    property    FileSize: int64 read GetFileSize;
    property    FilePos: int64 read GetFilePos;

    procedure   AssignFile;
    procedure   Reset( FileMode: integer );
    procedure   BlockRead; overload;
    procedure   BlockRead( var Transfered: integer ); overload;
    procedure   BlockWrite; overload;
    procedure   BlockWrite( var Transfered: integer ); overload;
    procedure   Seek( NewPosition: int64 );
    procedure   CloseFile;
  end;

implementation

uses
  SysUtils;


constructor TARBC.Create( Buffer: pointer; BufferSize: integer );
begin
     inherited Create;
     if (Buffer <> nil) and (BufferSize > 0) then
     begin
          hBuffer:= Buffer;
          hSize:= BufferSize;
          hOpened:= FALSE;
          hFileName:= '';
     end
     else
     begin
          raise Exception.Create('Null buffer pointer assigment, or size of buffer is too small.');
     end;
end;

destructor TARBC.Destroy;
begin
     if hOpened then CloseFile;
     inherited Destroy;
end;

function TARBC.GetIOResult: integer;
begin
     Result:= System.IOResult;
end;

function TARBC.GetFileName: string;
begin
     Result:= hFileName;
end;

procedure TARBC.SetFileName( FN: string );
begin
     hFileName:= FN;
end;

function TARBC.GetFileSize: int64;
begin
     Result:= System.FileSize(F);
end;

function TARBC.GetFilePos: int64;
begin
     Result:= System.FilePos(F);
end;

procedure TARBC.AssignFile;
begin
     if hOpened then CloseFile;
     System.AssignFile(F, hFileName);
end;

procedure TARBC.Reset( FileMode: integer );
begin
     System.FileMode:= FileMode and $FF;
     System.Reset(F);
end;

procedure TARBC.BlockRead;
begin
     System.BlockRead(F, hBuffer^, hSize);
end;

procedure TARBC.BlockRead( var Transfered: integer );
begin
     System.BlockRead(F, hBuffer^, hSize, Transfered);
end;

procedure TARBC.BlockWrite;
begin
     System.BlockWrite(F, hBuffer^, hSize);
end;

procedure TARBC.BlockWrite( var Transfered: integer );
begin
     System.BlockWrite(F, hBuffer^, hSize, Transfered);
end;

procedure TARBC.Seek( NewPosition: int64 );
begin
     if hOpened then System.Seek(F, NewPosition);
end;

procedure TARBC.CloseFile;
begin
     if hOpened then System.CloseFile(F);
     hOpened:= FALSE;
end;



end.