unit ComponentTest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, FilesystemDialogsDefs, StdCtrls, XPMan, ExtCtrls;

type
  TFormFDD = class(TForm)
    Button1: TButton;
    Edit1: TEdit;
    Memo1: TMemo;
    FilesystemDialog1: TFilesystemDialog;
    GroupBox2: TGroupBox;
    Panel1: TPanel;
    Button2: TButton;
    Button3: TButton;
    GroupBox4: TGroupBox;
    GroupBox5: TGroupBox;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    Button8: TButton;
    FilesystemDialog2: TFilesystemDialog;
    FilesystemDialog3: TFilesystemDialog;
    Button9: TButton;
    Button10: TButton;
    Edit2: TEdit;
    Label2: TLabel;
    CheckBox1: TCheckBox;
    Edit3: TEdit;
    Label1: TLabel;
    Label3: TLabel;
    Edit4: TEdit;
    Button11: TButton;
    Button12: TButton;
    XPManifest1: TXPManifest;
    CheckBoxOfNotModal: TCheckBox;
    CheckBoxOfNoClose: TCheckBox;
    CheckBoxOfStayOnTop: TCheckBox;
    PanelParent: TPanel;
    CheckBoxOfIntegrate: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure Button10Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure Button8Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button11Click(Sender: TObject);
    procedure Button12Click(Sender: TObject);
    procedure WndProc(var Message: TMessage); override;
    procedure CheckBoxOfNotModalClick(Sender: TObject);
    procedure SetButtons;
  private
    { Private declarations }
    fHookProc: OFNHookProc;
  public
    { Public declarations }
  end;

    function OFNHookProc(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT stdcall;

var
  FormFDD: TFormFDD;

implementation

{$R *.dfm}

//* In this demo when not using the component the .dll is loaded every time here for demonstration purposes, it looses great
//* amount of speed as the form is re-created every time, normally you have to do this only on program start and use FreeLibrary()
//* when you exit your program!

procedure TFormFDD.Button1Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  OpenDialog: t_OpenDialog;
  Params: TOpenDialogParams;
  Results: TOpenDialogResults;
  pName: PChar;
  Success: Boolean;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of OpenDialog()
    OpenDialog := GetProcAddress(LibHandle, PChar(NAME_FD_OpenDialog));

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.FileName := PChar(Edit1.Text);
    Params.Filters := PChar(Edit3.Text);
    Params.FilterIndex := StrToInt(Edit4.Text);

    //* Multi or single
    if CheckBox1.Checked
        then Params.Options := Params.Options + [ofAllowMultiSelect]
        else Params.Options := Params.Options - [ofAllowMultiSelect];

    //* Modal or not (the NOT modal form is the suggested methode!)
    //* This requires a Handle which to the message will be sent and/or a HookProc
    //* This is needed to free the resources at the session end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNotModal];

    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNoClose];

    if CheckBoxOfStayOnTop.Checked
        then Params.Options := Params.Options + [ofStayOnTop]
        else Params.Options := Params.Options - [ofStayOnTop];

    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
    end else Params.Options := Params.Options - [ofIntegrate];

    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9C-F2ED-4BF7-B90E-0831BABA3093}');

    //* The app isn't commercial else you need a license
    Params.AllowTIFFLZW := True;

    ZeroMemory(@Results, SizeOf(Results));
    //* Get mem for result
    GetMem(Results.FileName, FD_MAX_PATH);
    Results.FileNameSize := FD_MAX_PATH;
    //* Get mem for results 100 files max
    GetMem(Results.FileNames, FD_MAX_PATH * StrToInt(Edit2.Text));
    Results.FileNamesSize := FD_MAX_PATH * StrToInt(Edit2.Text);

    //* Call OpenDialog()
    Success := OpenDialog(Params, Results);

    //* Use result
    if Results.Successfull then begin

        //* Single file
        Edit1.Text := Results.FileName;

        //* Extract files (multiple) ending with a double null
        Memo1.Clear;
        pName := Results.FileNames;
        while pName^ <> #0 do begin
            Memo1.Lines.Append(pName);
            // Point to the next filename
            Inc(pName, Length(pName) + 1);
        end;

        //* Filter selected
        Edit4.Text := IntToStr(Params.FilterIndex);

    end;

    //* Free memory
    if NOT CheckBoxOfNotModal.Checked then begin
        FreeMem(Results.FileName);
        FreeMem(Results.FileNames);

        //* Release FilesystemDialogs library
        FreeLibrary(LibHandle);
    end;
    //* Memory leak!
    //* You have to FreeMem() and FreeLibrary() in your application end somewhere
    //* if you are using the ofNotModal mode for example in .FormDestroy()

end;

procedure TFormFDD.Button3Click(Sender: TObject);
var
  i: Cardinal;
begin

    //* Set initial dir and filename
    FilesystemDialog1.FileName := Edit1.Text;

    //* Set max. results for multi
    FilesystemDialog1.MaxFiles := StrToInt(Edit2.Text);

    //* Multi or single
    if CheckBox1.Checked
        then FilesystemDialog1.Options := FilesystemDialog1.Options + [ofAllowMultiSelect]
        else FilesystemDialog1.Options := FilesystemDialog1.Options - [ofAllowMultiSelect];

    //* Modal or not (the NOT modal form is the suggested methode!)
    //* This requires a Handle which to the message will be sent and a HookProc that will be invoked at the end
    if CheckBoxOfNotModal.Checked then begin
        FilesystemDialog1.Options := FilesystemDialog1.Options + [ofNotModal];
        FilesystemDialog1.Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        FilesystemDialog1.Params.HookProc := fHookProc;
    end else FilesystemDialog1.Options := FilesystemDialog1.Options - [ofNotModal];

    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        FilesystemDialog1.Options := FilesystemDialog1.Options + [ofNoClose];
        FilesystemDialog1.Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        FilesystemDialog1.Params.HookProc := fHookProc;
    end else FilesystemDialog1.Options := FilesystemDialog1.Options - [ofNoClose];

    if CheckBoxOfStayOnTop.Checked
        then FilesystemDialog1.Options := FilesystemDialog1.Options + [ofStayOnTop]
        else FilesystemDialog1.Options := FilesystemDialog1.Options - [ofStayOnTop];

    //* Filters
    FilesystemDialog1.Filter := Edit3.Text;
    FilesystemDialog1.FilterIndex := StrToInt(Edit4.Text);

    //* Open window
    if FilesystemDialog1.Execute then begin

        //* Single filename
        Edit1.Text := FilesystemDialog1.FileName;

        //* Add to list (single or multiple file names)
        Memo1.Clear;
        for i := 0 to FilesystemDialog1.Files.Count - 1
            do Memo1.Lines.Append(FilesystemDialog1.Files[i]);

        //* Filter selected
        Edit4.Text := IntToStr(FilesystemDialog1.FilterIndex);

    end;
    
end;

procedure TFormFDD.Button9Click(Sender: TObject);
begin

    //* Set initial dir and filename
    FilesystemDialog2.FileName := Edit1.Text;

    //* Filters
    FilesystemDialog2.Filter := Edit3.Text;
    FilesystemDialog2.FilterIndex := StrToInt(Edit4.Text);    

    //* Modal or not (the NOT modal form is the suggested methode!)
    //* This requires a Handle which to the message will be sent and a HookProc that will be invoked at the end
    if CheckBoxOfNotModal.Checked then begin
        FilesystemDialog2.Options := FilesystemDialog2.Options + [ofNotModal];
        FilesystemDialog2.Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        FilesystemDialog2.Params.HookProc := fHookProc;
    end else FilesystemDialog2.Options := FilesystemDialog2.Options - [ofNotModal];

    if CheckBoxOfStayOnTop.Checked
        then FilesystemDialog1.Options := FilesystemDialog1.Options + [ofStayOnTop]
        else FilesystemDialog1.Options := FilesystemDialog1.Options - [ofStayOnTop];

    //* Execute
    if FilesystemDialog2.Execute then begin

        //* User selected filename
        Edit1.Text := FilesystemDialog2.FileName;

        Memo1.Clear;
        Memo1.Lines.Append(FilesystemDialog2.Files.Text);

        //* Filter selected
        Edit4.Text := IntToStr(FilesystemDialog2.FilterIndex);        

    end;

end;

procedure TFormFDD.Button10Click(Sender: TObject);
begin

    //* Set initial dir
    FilesystemDialog3.InitialDir := ExtractFilePath(Edit1.Text);

    if CheckBoxOfStayOnTop.Checked
        then FilesystemDialog1.Options := FilesystemDialog1.Options + [ofStayOnTop]
        else FilesystemDialog1.Options := FilesystemDialog1.Options - [ofStayOnTop];

    FilesystemDialog3.DialogType := fsdtFolderDialog;

    //* Execute
    if FilesystemDialog3.Execute then begin

        //* User selected folder name
        Edit1.Text := FilesystemDialog3.FileName;

        Memo1.Clear;
        Memo1.Lines.Append(FilesystemDialog3.InitialDir);

    end;

end;

procedure TFormFDD.Button7Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  SaveDialog: t_SaveDialog;
  Params: TOpenDialogParams;
  Results: TOpenDialogResults;
  pName: PChar;
  Success: Boolean;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of SaveDialog()
    SaveDialog := GetProcAddress(LibHandle, PChar(NAME_FD_SaveDialog));

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.FileName := PChar(Edit1.Text);
    Params.Filters := PChar(Edit3.Text);
    Params.FilterIndex := StrToInt(Edit4.Text);

    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9D-F2ED-4BF7-B90E-0831BABA3093}');
                                
    //* Modal or not (the NOT modal form is the suggested methode!)
    //* This requires a Handle which to the message will be sent and a HookProc that will be invoked at the end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNotModal];

    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNoClose];

    if CheckBoxOfStayOnTop.Checked
        then Params.Options := Params.Options + [ofStayOnTop]
        else Params.Options := Params.Options - [ofStayOnTop];

    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
    end else Params.Options := Params.Options - [ofIntegrate];        

    //* The app isn't commercial else you need a license
    Params.AllowTIFFLZW := True;

    ZeroMemory(@Results, SizeOf(Results));
    //* Get mem for result
    GetMem(Results.FileName, FD_MAX_PATH);
    Results.FileNameSize := FD_MAX_PATH;

    //* Get mem for result
    GetMem(Results.FileNames, FD_MAX_PATH);
    Results.FileNamesSize := FD_MAX_PATH;

    //* Call OpenDialog()
    Success := SaveDialog(Params, Results);

    //* Use result
    if Results.Successfull then begin

        //* Single file
        Edit1.Text := Results.FileName;

        Memo1.Clear;
        Memo1.Lines.Append(Results.FileNames);

        //* Filter selected
        Edit4.Text := IntToStr(Params.FilterIndex);        

    end;

    //* Free memory
    if NOT CheckBoxOfNoClose.Checked then begin
        FreeMem(Results.FileName);

        //* Release FilesystemDialogs library
        FreeLibrary(LibHandle);
    end;
    //* Memory leak!
    //* You have to FreeMem() and FreeLibrary() in your application end somewhere
    //* if you are using the ofNotModal mode for example in .FormDestroy()    

end;

procedure TFormFDD.Button8Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  FolderDialog: t_FolderDialog;
  Params: TOpenDialogParams;
  Results: TOpenDialogResults;
  pName: PChar;
  Success: Boolean;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of FolderDialog()
    FolderDialog := GetProcAddress(LibHandle, PChar(NAME_FD_FolderDialog));

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.InitialDir := PChar(ExtractFilePath(Edit1.Text));

    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9D-F2ED-4BF7-B90E-0831BABA3093}');

    ZeroMemory(@Results, SizeOf(Results));
    //* Get mem for result
    GetMem(Results.FileName, FD_MAX_PATH);
    Results.FileNameSize := FD_MAX_PATH;

    GetMem(Results.FileNames, FD_MAX_PATH);
    Results.FileNamesSize := FD_MAX_PATH;

    //* Modal or not (the NOT modal form is the suggested methode!)
    //* This requires a Handle which to the message will be sent and/or a HookProc
    //* This is needed to free the resources at the session end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNotModal];

    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := Handle;
        fHookProc := OFNHookProc;
        Params.HookProc := fHookProc;
    end else Params.Options := Params.Options - [ofNoClose];    

    if CheckBoxOfStayOnTop.Checked
        then Params.Options := Params.Options + [ofStayOnTop]
        else Params.Options := Params.Options - [ofStayOnTop];

    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
    end else Params.Options := Params.Options - [ofIntegrate];        

    //* Call OpenDialog()
    Success := FolderDialog(Params, Results);

    //* Use result
    if Results.Successfull then begin

        //* Single file
        Edit1.Text := Results.FileName;

        Memo1.Clear;
        Memo1.Lines.Append(Results.FileNames);

    end;

    //* Free memory
    if NOT CheckBoxOfNotModal.Checked then begin
        FreeMem(Results.FileName);

        //* Release FilesystemDialogs library
        FreeLibrary(LibHandle);
    end;
    //* Memory leak!
    //* You have to FreeMem() and FreeLibrary() in your application end somewhere
    //* if you are using the ofNotModal mode for example in .FormDestroy()

end;

procedure TFormFDD.Button2Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  OpenDialog: t_GetFileName;
  Params: tagOFN;
  pName: PChar;
  Success: Boolean;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of OpenDialog()
    OpenDialog := GetProcAddress(LibHandle, PChar(NAME_FD_GetOpenFileNameA));

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    GetMem(Params.lpstrFile, FD_MAX_PATH * StrToInt(Edit2.Text));
    Params.nMaxFile := FD_MAX_PATH * StrToInt(Edit2.Text);
    CopyMemory(Params.lpstrFile, PChar(Edit1.Text + #0), Length(Edit1.Text) + 1);

    GetMem(Params.lpstrFilter, Length(Edit3.Text) + 1);
    CopyMemory(Params.lpstrFilter, PChar(Edit3.Text + #0), Length(Edit3.Text) + 1);
    Params.nFilterIndex := StrToInt(Edit4.Text);

    //* Multi or single
    if CheckBox1.Checked
        then Params.Flags := OFN_ALLOWMULTISELECT;

    //* Call OpenDialog()
    Success := OpenDialog(Params);

    //* Use result
    if Success then begin

        //* Single file
        Edit1.Text := Params.lpstrFile;

        //* Extract files (multiple) ending with a double null
        Memo1.Clear;
        pName := Params.lpstrFile;
        while pName^ <> #0 do begin
            Memo1.Lines.Append(pName);
            // Point to the next filename
            Inc(pName, Length(pName) + 1);
        end;

        //* Filter selected
        Edit4.Text := IntToStr(Params.nFilterIndex);

    end;

    //* Free memory
    FreeMem(Params.lpstrFile);
    FreeMem(Params.lpstrFilter);

    //* Release FilesystemDialogs library
    FreeLibrary(LibHandle);

end;

procedure TFormFDD.Button5Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  SaveDialog: t_GetFileName;
  Params: tagOFN;
  pName: PChar;
  Success: Boolean;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of OpenDialog()
    SaveDialog := GetProcAddress(LibHandle, PChar(NAME_FD_GetSaveFileNameA));

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    GetMem(Params.lpstrFile, FD_MAX_PATH * StrToInt(Edit2.Text));
    Params.nMaxFile := FD_MAX_PATH * StrToInt(Edit2.Text);
    CopyMemory(Params.lpstrFile, PChar(Edit1.Text + #0), Length(Edit1.Text) + 1);

    GetMem(Params.lpstrFilter, Length(Edit3.Text) + 1);
    CopyMemory(Params.lpstrFilter, PChar(Edit3.Text + #0), Length(Edit3.Text) + 1);

    Params.nFilterIndex := StrToInt(Edit4.Text);

    //* Call SaveDialog()
    Success := SaveDialog(Params);

    //* Use result
    if Success then begin

        //* Single file
        Edit1.Text := Params.lpstrFile;

        //* Extract files (multiple) ending with a double null
        Memo1.Clear;
        pName := Params.lpstrFile;
        while pName^ <> #0 do begin
            Memo1.Lines.Append(pName);
            // Point to the next filename
            Inc(pName, Length(pName) + 1);
        end;

        //* Filter selected
        Edit4.Text := IntToStr(Params.nFilterIndex);        

    end;

    //* Free memory
    FreeMem(Params.lpstrFile);
    FreeMem(Params.lpstrFilter);

    //* Release FilesystemDialogs library
    FreeLibrary(LibHandle);

end;

procedure TFormFDD.Button6Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  SaveDialog: t_GetFileName;
  Params: tagOFN;
  pName: PWideChar;
  Success: Boolean;
  wFileName: PWideChar;
  wFNLength: Cardinal;
  wFilters: PWideChar;
  wFLength: Cardinal;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of OpenDialog()
    SaveDialog := GetProcAddress(LibHandle, PChar(NAME_FD_GetSaveFileNameW));

    wFNLength := Length(Edit1.Text) * 2 + 2;
    GetMem(wFileName, wFNLength);
    StringToWideChar(Edit1.Text, wFileName, wFNLength);

    wFLength := Length(Edit3.Text) * 2 + 2;
    GetMem(wFilters, wFLength);
    StringToWideChar(Edit3.Text, wFilters, wFLength);

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    GetMem(Params.lpstrFile, FD_MAX_PATH * StrToInt(Edit2.Text));
    Params.nMaxFile := FD_MAX_PATH * StrToInt(Edit2.Text);
    CopyMemory(Params.lpstrFile, wFileName, wFNLength);

    GetMem(Params.lpstrFilter, wFLength);
    CopyMemory(Params.lpstrFilter, wFilters, wFLength);

    FreeMem(wFileName);
    FreeMem(wFilters);

    Params.nFilterIndex := StrToInt(Edit4.Text);

    //* Call SaveDialog()
    Success := SaveDialog(Params);

    //* Use result
    if Success then begin

        //* Single file
        Edit1.Text := WideCharToString(PWideChar(Params.lpstrFile));

        //* Extract files (multiple) ending with a double null
        Memo1.Clear;
        pName := PWideChar(Params.lpstrFile);
        while pName^ <> #0 do begin
            Memo1.Lines.Append(WideCharToString(pName));
            // Point to the next filename
            Inc(pName, Length(pName) + 1);
        end;

        //* Filter selected
        Edit4.Text := IntToStr(Params.nFilterIndex);        

    end;

    //* Free memory
    FreeMem(Params.lpstrFile);
    FreeMem(Params.lpstrFilter);

    //* Release FilesystemDialogs library
    FreeLibrary(LibHandle);

end;

procedure TFormFDD.Button4Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  OpenDialog: t_GetFileName;
  Params: tagOFN;
  pName: PWideChar;
  Success: Boolean;
  wFileName: PWideChar;
  wFNLength: Cardinal;
  wFilters: PWideChar;
  wFLength: Cardinal;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of OpenDialog()
    OpenDialog := GetProcAddress(LibHandle, PChar(NAME_FD_GetOpenFileNameW));

    wFNLength := Length(Edit1.Text) * 2 + 2;
    GetMem(wFileName, wFNLength);
    StringToWideChar(Edit1.Text, wFileName, wFNLength);

    wFLength := Length(Edit3.Text) * 2 + 2;
    GetMem(wFilters, wFLength);
    StringToWideChar(Edit3.Text, wFilters, wFLength);

    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    GetMem(Params.lpstrFile, FD_MAX_PATH * StrToInt(Edit2.Text));
    Params.nMaxFile := FD_MAX_PATH * StrToInt(Edit2.Text);
    CopyMemory(Params.lpstrFile, wFileName, wFNLength);

    GetMem(Params.lpstrFilter, wFLength);
    CopyMemory(Params.lpstrFilter, wFilters, wFLength);

    FreeMem(wFileName);
    FreeMem(wFilters);

    Params.nFilterIndex := StrToInt(Edit4.Text);

    //* Multi or single
    if CheckBox1.Checked
        then Params.Flags := OFN_ALLOWMULTISELECT;

    //* Call OpenDialog()
    Success := OpenDialog(Params);

    //* Use result
    if Success then begin

        //* Single file
        Edit1.Text := WideCharToString(PWideChar(Params.lpstrFile));

        //* Extract files (multiple) ending with a double null
        Memo1.Clear;
        pName := PWideChar(Params.lpstrFile);
        while pName^ <> #0 do begin
            Memo1.Lines.Append(WideCharToString(pName));
            // Point to the next filename
            Inc(pName, Length(pName) + 1);
        end;

        //* Filter selected
        Edit4.Text := IntToStr(Params.nFilterIndex);        

    end;

    //* Free memory
    FreeMem(Params.lpstrFile);
    FreeMem(Params.lpstrFilter);

    //* Release FilesystemDialogs library
    FreeLibrary(LibHandle);

end;

procedure TFormFDD.Button11Click(Sender: TObject);
begin
    Memo1.Clear;
    if FilesystemDialog1.Settings
        then Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): OK.')
        else Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): ERROR.');
end;

procedure TFormFDD.Button12Click(Sender: TObject);
var
  //* Dialog vars
  LibHandle: THandle;
  Settings: t_FilesystemDialogsSettings;
  Success: Boolean;
begin

    //* Load FilesystemDialogs.dll
    LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));

    //* Get Process address of SaveDialog()
    Settings := GetProcAddress(LibHandle, PChar(NAME_FD_Settings));

    //* Call FilesystemDialogs_SettingsWindow()
    Success := Settings;

    //* Use result
    Memo1.Clear;
    if Success
        then Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): OK.')
        else Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): ERROR.');

    //* Release FilesystemDialogs library
    FreeLibrary(LibHandle);

end;

function OFNHookProc(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT stdcall;
type
  POpenDialogResults = ^TOpenDialogResults;
var
  Res: POpenDialogResults;
  Text: String;
begin

    //* Sent results
    if Msg = FDM_RESULTS then begin

        Res := POpenDialogResults(wParam);

        //* Extract first! filename
        Text := PChar(Res.FileName);

        FormFDD.Memo1.Lines.Add(Text);

    end;

    //* Free the memory as it is not needed anymore
    if Msg = FDM_FREEMEM then begin

        //* Free memory
        FreeMem(Res.FileName);
        FreeMem(Res.FileNames);
        // or
        // FreeTOpenDialogResults(Res);
    end;
    
end;

procedure TFormFDD.WndProc(var Message: TMessage);
begin
    try
        //* Process messages like this in ofNotModal and ofNoClose mode
        case Message.Msg of
            FDM_WINDOW_HANDLE: ShowMessage('FDM_WINDOW_HANDLE');
            FDM_CLOSING: ShowMessage('FDM_CLOSING');
            FDM_FREEMEM: ShowMessage('FDM_FREEMEM');
            FDM_RESULTS: ShowMessage('FDM_RESULTS');
        else inherited WndProc(Message);
        end;
    except
        //*
    end;
end;

procedure TFormFDD.CheckBoxOfNotModalClick(Sender: TObject);
begin
    SetButtons;
end;

procedure TFormFDD.SetButtons;
begin
    if CheckBoxOfNotModal.Checked
    OR CheckBoxOfNoClose.Checked
    OR CheckBoxOfStayOnTop.Checked
    OR CheckBoxOfIntegrate.Checked
    then begin
        Button2.Enabled := False;
        Button4.Enabled := False;
        Button5.Enabled := False;
        Button6.Enabled := False;
    end else begin
        Button2.Enabled := True;
        Button4.Enabled := True;
        Button5.Enabled := True;
        Button6.Enabled := True;
    end;

end;

end.
