/*********************************************************************************************************************************
//*                                                                                                                              *
//*     Filesystem Dialogs  3delite 2006-2007                                                                                   *
//*     See FilesystemDialogs.txt for details                                                                                    *
//*                                                                                                                              *
//* Two licenses are available if you like this library:                                                                         *
//* Shareware License: 50 Euros                                                                                                  *
//* Commercial License: 500 Euros                                                                                                *
//*                                                                                                                              *
//*     http://www.3delite.hu/Object Pascal Developer Resources/filesystemdialogs.html                                           *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//*******************************************************************************************************************************/

#include <wtypes.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef FILESYSTEMDIALOGSDEF
#define FILESYSTEMDIALOGSDEF(f) WINAPI f
#endif

//* .dll file names
#define FILENAME_DLL_FD                 "FilesystemDialogs.dll"
#define FILENAME_DLL_VRM                "VangaRengiMangaro.dll"

//* Exported function names
#define NAME_OpenDialog                 "FilesystemDialogs_OpenDialog"
#define NAME_SaveDialog                 "FilesystemDialogs_SaveDialog"
#define NAME_FolderDialog               "FilesystemDialogs_FolderDialog"
#define NAME_FilesystemDialogsSettings  "FilesystemDialogs_SettingsWindow"
//* comdlg32.dll compatibility connection
#define NAME_FD_GetOpenFileNameA        "FilesystemDialogs_GetOpenFileNameA"
#define NAME_FD_GetOpenFileNameW        "FilesystemDialogs_GetOpenFileNameW"
#define NAME_FD_GetSaveFileNameA        "FilesystemDialogs_GetSaveFileNameA"
#define NAME_FD_GetSaveFileNameW        "FilesystemDialogs_GetSaveFileNameW"
//* VangaRengiMangaro.dll's function
#define NAME_FD_GetPrevProcs            "VangaRengiMangaro_GetPrevProcs"

#define FD_MAX_PATH                     MAX_PATH * 30

//* Pointer to TOpenDialogResults is in wParam
//* Pointer to TOpenDialogParams is in lParam
//* You get the messages in the hookproc and to the specified Parent window handle
#define WM_FD_BASE                      2 * WM_USER + 1

//* Filesystem Dialogs results
//* Pointer to TOpenDialogResults in wParam
//* You need to specify the ParentWindow also
#define WM_FD_RESULTS                   WM_FD_BASE + 1

//* Load settings
#define FDM_LOADSETTINGS      			FDM_BASE + 2;

//* Free memory that you have allocated for the dialog('s params, results)
#define FDM_FREEMEM           			FDM_BASE + 3;

//* The dialog form is closing
#define FDM_CLOSING           			FDM_BASE + 4;

//* The dialog form's window handle in wParam
//* Pointer to TOpenDialogParams is in lParam
#define FDM_WINDOW_HANDLE     			FDM_BASE + 5;

//* Realing the integrated form to the parent
#define FDM_REALIGN_INTEGRATED_WINDOW 	FDM_BASE + 6;


//* from Dialogs.pas
enum
{
    ofReadOnly, ofOverwritePrompt, ofHideReadOnly,
    ofNoChangeDir, ofShowHelp, ofNoValidate, ofAllowMultiSelect,
    ofExtensionDifferent, ofPathMustExist, ofFileMustExist, ofCreatePrompt,
    ofShareAware, ofNoReadOnlyReturn, ofNoTestFileCreate, ofNoNetworkButton,
    ofNoLongNames, ofOldStyleDialog, ofNoDereferenceLinks, ofEnableIncludeNotify,
    ofEnableSizing, ofDontAddToRecent, ofForceShowHidden, ofNotModal, ofNoClose,
    ofIntegrate, ofStayOnTop
}; // TOpenOption

//* from CommDlg.pas
UINT FILESYSTEMDIALOGSDEF(OFNHookProc)(HWND Wnd, UINT Msg, WPARAM wParam, LPARAM lParam);  //* stdcall

//* Filesystem dialogs types
enum
{
    fsdtOpenDialog,
    fsdtSaveDialog,
    fsdtFolderDialog
}; // TFSDialogType

//* TGUID
typedef struct {
    LongWord D1;
    Word D2;
    Word D3;
    Byte D4[08];
} TGUID;

//* Filesystem dialog Open file/Save file/Folder dialog Params
typedef struct {
    PChar DefaultExt;          //* Default extension
    PChar FileName;            //* Default file name for Open/Save and Initial dir
    PChar Filter;              //* Filters eg.: 'Image files (*.bmp,*.jpg)|*.bmp;*.jpg|All files|*.*'
    Int FilterIndex;           //* Filter to use default, 1-first, 2-second...
    Int HelpContext;           //* Help context (not yet implemented)
    PChar InitialDir;          //* Initial folder, this is extracted first from FileName, if no FileName specified settings are used to decide which folder to use
    Int Options;               //* TOpenOptions: Dialog options (not all implemented so far)
    Int OptionsEx;             //* Extra Dialog options (not implemented, probably never will)
    PChar Title;               //* Title to display as window's caption
    HWND ParentWindow;         //* Parent window handle (needed for HookProc)
    TGUID DialogGUID;          //* Specifically identificate the dialog (you can get the same dialog always with this flag)
    OFNHookProc HookProc;      //* Hookproc (for not modal mode, ParentWindow handle is also needed)
    TFSDialogType DialogType;  //* Type of the dialog: Open/Save/Folder
    LongBool AllowTIFFLZW;     //* Allow LZW coded TIF image thumbnails (using these formats requires a license)
    //FileSize: Int64;			//* Estimated file size (will warn if won't fit in the selected directory before performing any saving)
    //MD5: String;				//* MD5 value of the file to save (compare to exisisting files)
    //User: Pointer;            //* User variable
} TOpenDialogParams;

//* Filesystem dialog result
typedef struct {
    BOOL Successfull;          //* True: User pressed the Open/Save, False: User pressed Cancel, closed the window or error (there is a fallback to comdlg32.dll though)
    PChar FileName;            //* The filename the user selected (only one) ending with a double null
    QWORD FileNameSize;        //* FileName size in bytes (should be always at least MAX_PATH)
    DWORD FileCount;           //* Number of files, if user selected multiple files then FileCount > 1
    PChar FileNames;           //* List of files in format "C:\Dir\Data.txt" separated by null characters ending with a double null
    QWORD FileNamesSize;       //* FileNames size in bytes (it's up to you to validate last entry if all selected filenames don't fit into the FileNames)
    Int FilterIndex;           //* Filter's index the user selected 1-first, 2-second, etc...
} TOpenDialogResults;

//* Exported functions
BOOL FILESYSTEMDIALOGSDEF(t_OpenDialog)(TOpenDialogParams Params, TOpenDialogResults Results);
BOOL FILESYSTEMDIALOGSDEF(t_SaveDialog)(TOpenDialogParams Params, TOpenDialogResults Results);
BOOL FILESYSTEMDIALOGSDEF(t_FolderDialog)(TOpenDialogParams Params, TOpenDialogResults Results);
BOOL FILESYSTEMDIALOGSDEF(t_FilesystemDialogsSettings)();

BOOL FILESYSTEMDIALOGSDEF(t_InitTOpenDialogParams)(TOpenDialogParams Struct, QWORD MaxFiles = 1000);
BOOL FILESYSTEMDIALOGSDEF(t_FreeTOpenDialogParams)(TOpenDialogParams Struct);
BOOL FILESYSTEMDIALOGSDEF(t_InitTOpenDialogResults)(TOpenDialogResults Res, QWORD MaxFiles = 1000);
BOOL FILESYSTEMDIALOGSDEF(t_FreeTOpenDialogResults)(TOpenDialogResults, Struct);

BOOL FILESYSTEMDIALOGSDEF(t_GetFileName)(tagOFN OpenFileParams);

ByteBool FILESYSTEMDIALOGSDEF(t_GetPrevProcs)(void *pGetOpenFileNameA, void *pGetOpenFileNameW, void *pGetSaveFileNameA, void *pGetSaveFileNameW); // register;

#ifdef __cplusplus
}
#endif

#endif
