unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Buttons, id3v2;

type
  TLoadThread = class(TThread)
  protected
   procedure Execute; override;
  public
   constructor Create;
  end;

  TSaveThread = class(TThread)
  protected
   procedure Execute; override;
  public
   constructor Create;
  end;

  TForm1 = class(TForm)
    FramesListBox: TListBox;
    Label1: TLabel;
    LoadTag: TButton;
    GroupBox1: TGroupBox;
    OptionURev: TCheckBox;
    TagContentListBox: TListBox;
    Label2: TLabel;
    FrameContentListBox: TListBox;
    Label3: TLabel;
    FrameDataListBox: TListBox;
    Label4: TLabel;
    OptionCRC: TCheckBox;
    id3v2tagfile: TEdit;
    Label5: TLabel;
    ProgressBar1: TProgressBar;
    Timer1: TTimer;
    Timer2: TTimer;
    SaveTag: TButton;
    SyncFrame: TButton;
    UnsyncFrame: TButton;
    ProgressText: TLabel;
    DeleteFrame: TButton;
    CompressFrame: TButton;
    DecompressFrame: TButton;
    ExpTag: TButton;
    UnsyncTag: TButton;
    CRCTag: TButton;
    PadTag: TButton;
    Label6: TLabel;
    padSize: TEdit;
    OptionBypass: TCheckBox;
    OptionStandard: TCheckBox;
    Label7: TLabel;
    CompLevel: TComboBox;
    RemoveTag: TButton;
    Label8: TLabel;
    RevToSave: TEdit;
    addGEOBframe: TButton;
    openTag: TOpenDialog;
    openFile: TOpenDialog;
    openTagButton: TSpeedButton;
    UpdateTag: TButton;
    ExtTag: TButton;
    RestrictionValue: TEdit;
    RestrictionTag: TButton;
    procedure LoadTagClick(Sender: TObject);
    procedure FramesListBoxClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure SyncFrameClick(Sender: TObject);
    procedure UnsyncFrameClick(Sender: TObject);
    procedure SaveTagClick(Sender: TObject);
    procedure DeleteFrameClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CompressFrameClick(Sender: TObject);
    procedure DecompressFrameClick(Sender: TObject);
    procedure UnsyncTagClick(Sender: TObject);
    procedure ExpTagClick(Sender: TObject);
    procedure CRCTagClick(Sender: TObject);
    procedure ExtTagClick(Sender: TObject);
    procedure UpdateTagClick(Sender: TObject);
    procedure RestrictionTagClick(Sender: TObject);
    procedure PadTagClick(Sender: TObject);
    procedure CompLevelChange(Sender: TObject);
    procedure RemoveTagClick(Sender: TObject);
    procedure addGEOBframeClick(Sender: TObject);
    procedure openTagButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    loadThread : TLoadThread;
    saveThread : TSaveThread;
    procedure processTag;
    procedure processHeader;
  end;

function binBool(const Value: boolean):shortint;
// Returns the binary equivalent of the boolean value
//  (True = 1, False = 0)

function strBool(const Value: boolean):string;
// Returns the string equivalent of the boolean value

var
  Form1: TForm1;
  myTag : Tid3v2tag;
  temp : WORD;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
CompLevel.itemindex := 2;
end;

{******************************************************************************}
//functions for simplifying the display of tag/frame information

function binBool(const Value: boolean):shortint; begin
if Value then result := 1 else result := 0; end;

function strBool(const Value: boolean):string; begin
if Value then result := 'True' else result := 'False'; end;
{end***************************************************************************}

//Gets the main tag header information and displays it in the TagContentListBox listbox.
procedure TForm1.processTag;
var
I : cardinal;
begin
//Turn off the progress bar
timer1.enabled := false;
Timer1timer(self);
if myTag.isUnsynced then
 UnsyncTag.caption := 'Unsync All' else
  UnsyncTag.caption := 'Unsync None';
if myTag.isExperimental then
 ExpTag.caption := 'Exp. on' else
  ExpTag.caption := 'Exp. off';
if myTag.isCRC then
 CRCTag.caption := 'CRC-32 on' else
  CRCTag.caption := 'CRC-32 off';
RevToSave.text := inttostr(myTag.majorRev);
FramesListBox.clear;
for I := 1 to myTag.Frames.Count do
 FramesListBox.items.add(format('%d.) ''%s''', [I, myTag.Frames.getFrameID(I)]));
processHeader;
LoadTag.Enabled := true;
end;

procedure TForm1.processHeader;
begin
//Display Tag Info
TagContentListBox.clear;
TagContentListBox.items.add('TagErrorInt/Valid Tag:   ' + inttostr(temp) + ' / ' + inttostr(myTag.validTag));
TagContentListBox.items.add('Major/Minor Rev: ' + inttostr(myTag.MajorRev) + '.' + inttostr(myTag.MinorRev));
TagContentListBox.items.add('TagSetSize: ' + inttostr(myTag.tagSetSize));
//Create output for HeaderFlags
TagContentListBox.items.add(format('HeaderFlag: %%%d%d%d%d0000', [binBool(myTag.isUnsynced),
  binBool(myTag.isExtended), binBool(myTag.isExperimental), binBool(myTag.isFooter)]));
//Create output for ExtHeaderFlags
TagContentListBox.items.add(format('ExtHeaderFlags: %%0%d%d%d0000', [binBool(myTag.isUpdate),
  binBool(myTag.isCRC), binBool(myTag.isRestriction)]));
TagContentListBox.items.add('PadSize: ' + inttostr(myTag.padSize));
TagContentListBox.items.add('ExtendedHeaderSize: ' + inttostr(myTag.extendedHeaderSize));
TagContentListBox.items.add('CRCValue: $' + inttohex(myTag.CRCvalue, 8));
TagContentListBox.items.add('Restrictions: ' + inttostr(myTag.tagRestriction));
TagContentListBox.items.add(format('Changed: %s', [strBool(myTag.changed)]));
TagContentListBox.items.add(format('Read-Only: %s', [strBool(myTag.sourceReadOnly)]));
TagContentListBox.items.add('Sourcefile: ' + myTag.sourceFile);
TagContentListBox.items.add('SourceFileOffset: ' + inttostr(myTag.sourceFileOffset));
end;

{******************************************************************************}
//Seperate threads used to update the progress bar while the tag is being
//loaded/saved

procedure TLoadThread.execute;
begin
FreeOnTerminate := true;
temp := myTag.loadFromFile(Form1.id3v2tagfile.text);
Form1.processTag;
if temp > 255 then
 begin
 myTag.Frames.deleteAll;
 Form1.ProgressBar1.position := 0;
 Form1.ProgressText.caption := 'Load tag error #' + inttostr(temp);
 Form1.FramesListBox.clear;
 Form1.FrameContentListBox.clear;
 Form1.FrameDataListBox.clear;
 end;
end;

constructor TLoadThread.create;
begin
inherited Create(true);
Resume;
end;

procedure TSaveThread.execute;
begin
FreeOnTerminate := true;
temp := myTag.saveToFile('=', strtoint(Form1.RevToSave.text));
Form1.timer2.enabled := false;
Form1.Timer2timer(self);
if temp > 255 then
 begin
 Form1.ProgressBar1.position := 0;
 Form1.ProgressText.caption := 'Save tag error #' + inttostr(temp);
 end;
Form1.SaveTag.enabled := true;
end;

constructor TSaveThread.create;
begin
inherited Create(true);
Resume;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
ProgressBar1.position := myTag.progress;
ProgressText.Caption := myTag.progressInfo;
ProgressText.update;
end;

procedure TForm1.LoadTagClick(Sender: TObject);
begin
LoadTag.Enabled := false;
//myTag := Tid3v2tag.create;
myTag.options.highMinor := OptionURev.checked;
myTag.options.verifyCRC := OptionCRC.checked;
myTag.options.bypassUnsupported := OptionBypass.checked;
myTag.options.adhereToStandard := OptionStandard.checked;
myTag.options.compLevel := CompLevel.itemindex;
timer1.enabled := true;
loadThread := TLoadThread.create;
end;

procedure TForm1.Timer2Timer(Sender: TObject);
begin
ProgressBar1.position := myTag.progress;
ProgressText.Caption := myTag.progressInfo;
ProgressText.update;
end;

procedure TForm1.SaveTagClick(Sender: TObject);
begin
if myTag.loaded then
 begin
 SaveTag.Enabled := false;
 timer2.enabled := true;
 saveThread := TSaveThread.create;
 end;
end;
{end***************************************************************************}


{******************************************************************************}
//Display the information on the selected frame in the FrameContentListBox and
//show the raw content of the frame in hexidecimal and ascii form

procedure TForm1.FramesListBoxClick(Sender: TObject);
type
StreamPointer = array of byte;
var
tempString, asciiString : string;
Count, Remainder, LineCount, DataSize : cardinal;
myStreamP : ^StreamPointer;
tempByte : byte;
begin
FrameContentListBox.clear;
FrameContentListBox.items.add('FrameID: ''' + myTag.Frames.getFrameID(FramesListBox.itemindex+1)+ '''');
FrameContentListBox.items.add('Size: ' + inttostr(myTag.Frames.getSize(FramesListBox.itemindex+1)));
FrameContentListBox.items.add(format('Flags: %%0%d%d%d0000  %%0%d00%d%d%d%d',
 [binBool(myTag.Frames.isNowTagAlter(FramesListBox.itemindex+1)),
  binBool(myTag.Frames.isNowFileAlter(FramesListBox.itemindex+1)),
  binBool(myTag.Frames.isNowReadOnly(FramesListBox.itemindex+1)),
  binBool(myTag.Frames.isNowGrouped(FramesListBox.itemindex+1)),
  binBool(myTag.Frames.isNowCompressed(FramesListBox.itemindex+1)),
  binBool(myTag.Frames.isNowEncrypted(FramesListBox.itemindex+1)),
  binBool(myTag.Frames.isNowUnsynced(FramesListBox.itemindex+1)),
  binBool(myTag.Frames.isNowDataLength(FramesListBox.itemindex+1))]));
FrameContentListBox.items.add('SizeOfHeader: ' + inttostr(myTag.Frames.getSizeOfHeader(FramesListBox.itemindex+1)));
FrameContentListBox.items.add('Location: ' + inttostr(myTag.Frames.getLocation(FramesListBox.itemindex+1)));
//FrameContentListBox.items.add(format('DataLinked: %s',[strBool(myTag.Frames.isDataLinked(FramesListBox.ItemIndex+1))]));
FrameContentListBox.items.add('Data Length: ' + inttostr(myTag.Frames.getDataLength(FramesListBox.itemindex+1)));
FrameContentListBox.items.add('Encoding Method: ' + inttostr(myTag.Frames.getEncMethod(FramesListBox.itemindex+1)));
FrameContentListBox.items.add('Group ID: ' + inttostr(myTag.Frames.getGroupID(FramesListBox.itemindex+1)));

myStreamP := myTag.Frames.getFrameDataAddr(FramesListBox.itemindex+1);
DataSize := myTag.Frames.getSize(FramesListBox.itemindex+1);
Count := 0;
{Display Raw Frame Content in Hex and char}
FrameDataListBox.Clear;
while Count < DataSize do
 begin
 tempString := '';
 asciiString := '';
 if DataSize-Count >= 16 then
  Remainder := 16 else
   Remainder := DataSize-Count;

 for LineCount := 0 to Remainder-1 do
  begin
  tempByte := myStreamP^[LineCount+Count];
  tempString := tempString + inttohex(tempByte, 2) + ' ';
  if (tempByte > 31) and (tempByte < 255) then
   asciiString := asciiString + chr(tempByte) else
    asciiString := asciiString + chr(95);
  end;
 LineCount := Remainder-1;
 while LineCount < 15 do
  begin
  tempString := tempString + '   ';
  inc(LineCount);
  end;
 FrameDataListBox.items.add(tempString+asciiString);
 inc(Count, Remainder);
 end;
end;
{end***************************************************************************}

{******************************************************************************}
//Unsync/Sync & Compress/De-compress functions.
//Note: you cannot compress/decompress an Unsynced frame. You must Re-sync it
//first. Trying to do so will produce an Error #2.

procedure TForm1.UnsyncFrameClick(Sender: TObject);
var
I : integer;
begin
if FramesListBox.itemindex < 0 then exit;
I := myTag.Frames.unsync(FramesListBox.itemindex+1);
if I > 0 then
 showMessage('ERROR ' + inttostr(I)) else
  if (I = -1) then
   showMessage('Frame already unsynced.') else
    begin
    I := FramesListBox.itemIndex+1;
    myTag.frames.setSaveFlagBytes(I, 2, myTag.frames.SaveFlagBytes[I,2] or 2);
    end;
//Refresh display of frame data
FramesListBoxClick(self);
end;

procedure TForm1.SyncFrameClick(Sender: TObject);
var
I : integer;
begin
if FramesListBox.itemindex < 0 then exit;
I := myTag.Frames.sync(FramesListBox.itemindex+1);
if I > 0 then
 showMessage('ERROR ' + inttostr(I)) else
  if (I = -1) then
   showMessage('Frame already synced.') else
    begin
    I := FramesListBox.itemIndex+1;
    myTag.frames.setSaveFlagBytes(I, 2, myTag.frames.SaveFlagBytes[I,2] and 253);
    end;
FramesListBoxClick(self);
end;

procedure TForm1.CompressFrameClick(Sender: TObject);
var
I : integer;
begin
if FramesListBox.itemindex < 0 then exit;
I := myTag.Frames.compress(FramesListBox.itemindex+1, myTag.options.compLevel);
if I > 0 then
 showMessage('ERROR ' + inttostr(I)) else
  if (I = -1) then
   showMessage('Frame already compressed.') else
    begin
    I := FramesListBox.itemIndex+1;
    myTag.frames.setSaveFlagBytes(I, 2, myTag.frames.SaveFlagBytes[I,2] or 8);
    end;
FramesListBoxClick(self);
end;

procedure TForm1.DecompressFrameClick(Sender: TObject);
var
I : integer;
begin
if FramesListBox.itemindex < 0 then exit;
I := myTag.Frames.decompress(FramesListBox.itemindex+1);
if I > 0 then
 showMessage('ERROR ' + inttostr(I)) else
  if (I = -1) then
   showMessage('Frame already de-compressed.') else
    begin
    I := FramesListBox.itemIndex+1;
    myTag.frames.setSaveFlagBytes(I, 2, myTag.frames.SaveFlagBytes[I,2] and 247);
    end;
FramesListBoxClick(self);
end;
{end***************************************************************************}

//Delete the currently selected frame
procedure TForm1.DeleteFrameClick(Sender: TObject);
var
I, oldPos : integer;
begin
if FramesListBox.itemindex < 0 then exit;
if not(myTag.Frames.delete(FramesListBox.itemindex+1)) then
 showMessage('No frame found!') else
  begin
  oldPos := FramesListBox.itemindex;
  FramesListBox.clear;
  for I := 1 to myTag.Frames.Count do
   FramesListBox.items.add(format('%d.) ''%s''', [I, myTag.Frames.getFrameID(I)]));
  if (FramesListBox.Items.Count > 0) then
   begin
   if (oldPos+1 > FramesListBox.Items.Count) then
    FramesListBox.ItemIndex := FramesListBox.Items.Count-1 else
     FramesListBox.ItemIndex := oldPos;
   FramesListBoxClick(self);
   end else
   begin
   FrameContentListBox.clear;
   FrameDataListBox.clear;
   showMessage('Way to go genius! Now you''''ve got an empty tag!');
   end;
  end;
end;


{******************************************************************************}
//Unsynchronizes the entire tag. If this flag is set it overrides the
//individual frame level unsync flags and unsync's the entire tag.

procedure TForm1.UnsyncTagClick(Sender: TObject);
begin
if myTag.isUnsynced then
 begin
 myTag.setUnsync(false);
 UnsyncTag.caption := 'Unsync None';
 end else
 begin
 myTag.setUnsync(true);
 UnsyncTag.caption := 'Unsync All';
 end;
processHeader;
end;
{end***************************************************************************}


{******************************************************************************}
//Sets the Experimental flag on and off. Experimental tags are tags created
//with beta software. It is up to you to set this switch for your software.

procedure TForm1.ExpTagClick(Sender: TObject);
begin
if myTag.isExperimental then
 begin
 myTag.setExperimental(false);
 ExpTag.caption := 'Exp. off';
 end else
 begin
 myTag.setExperimental(true);
 ExpTag.caption := 'Exp. on';
 end;
processHeader;
end;
{end***************************************************************************}

{******************************************************************************}
//Turns on CRC-32 calculation. The CRC is calculated just before saving.

procedure TForm1.CRCTagClick(Sender: TObject);
begin
if myTag.isCRC then
 begin
 myTag.setCRC(false);
 CRCTag.caption := 'CRC-32 off';
 end else
 begin
 myTag.setCRC(true);
 CRCTag.caption := 'CRC-32 on';
 end;
processHeader;
end;
{end***************************************************************************}

{******************************************************************************}
//Sets the extended header. Turning off the extended header when any of the
//extended header flags are set (Update, CRC-32, or Tag Restrictions), has no
//effect.

procedure TForm1.ExtTagClick(Sender: TObject);
begin
if myTag.isExtended then
 begin
 myTag.setExtended(false);
 ExtTag.caption := 'Ext. Hdr. off';
 end else
 begin
 myTag.setExtended(true);
 ExtTag.caption := 'Ext. Hdr. on';
 end;
processHeader;
end;
{end***************************************************************************}

{******************************************************************************}
//Sets the tag update flag. A tag is an update whenever it is NOT the first
//tag in a file/stream. Useful for streaming, when you want to update the song
//name without having to resend all the other information.

procedure TForm1.UpdateTagClick(Sender: TObject);
begin
if myTag.isUpdate then
 begin
 myTag.setUpdate(false);
 UpdateTag.caption := 'Update off';
 end else
 begin
 myTag.setUpdate(true);
 UpdateTag.caption := 'Update on';
 end;
processHeader;
end;
{end***************************************************************************}

{******************************************************************************}
//Sets the tag restrictions flag and value. The tag restrictions are listed
//on id3.org (http://www.id3.org/id3v2.4.0-structure.txt)
//These are useful for portable devices that can't handle large tags/strings.
//This will guarentee that it is within the specified memory limits.

procedure TForm1.RestrictionTagClick(Sender: TObject);
begin
if myTag.isRestriction then
 begin
 myTag.setRestriction(false);
 RestrictionTag.caption := 'Restriction off';
 end else
 begin
 myTag.setRestriction(true);
 myTag.setRestrictionValue(strtoint(RestrictionValue.Text));
 RestrictionTag.caption := 'Restriction on';
 end;
processHeader;
end;
{end***************************************************************************}

{******************************************************************************}
//Sets the size of padding. Padding is used to fill out the empty space in a
//tag so that it doesn't have to be resized (along with the rest of the audio
//data) every time a change is made. It is common practice by taggers to align
//the padding to the cluster size, so that the padding doesn't actually use up
//any physical hard drive space. (This will be a future saving option)

procedure TForm1.PadTagClick(Sender: TObject);
begin
if not(myTag.isExtended) then
 myTag.setExtended(True);
myTag.padSize := strtoint(padSize.text);
processHeader;
end;
{end***************************************************************************}

procedure TForm1.CompLevelChange(Sender: TObject);
begin
myTag.Options.compLevel := CompLevel.itemIndex;
end;

{******************************************************************************}
//The kill() function supports the progress bar, but is not implemented here.
//If you want to remove an ID3v2 tag from a file without having to load the
//tag first, then use the killTag() function instead.
//eg: killTag(tagFileName, tagOffsetInFile);
//Also note that the killTag() function removes all tag versions.

procedure TForm1.RemoveTagClick(Sender: TObject);
var
I : byte;
begin
I := myTag.kill;
if I = 16 then
 showMessage('No tag found.') else
  if I > 0 then
   showMessage('Error removing tag.');
end;
{end***************************************************************************}

{******************************************************************************}
//Here is an example of manually setting the frame data, using setFrameData.

procedure TForm1.addGEOBframeClick(Sender: TObject);
const
demoString = #0'This is a demonstration of the GEOB frame.'#0;
var
tempStream : TStream;
tempString : String;
begin
if TagContentListBox.Items.count < 0 then exit;
if openFile.execute then
 begin
 if fileExists(openFile.FileName) then
  begin
  myTag.Frames.add('GEOB');
  tempStream := TMemoryStream.create;
  tempString := #0#0 + openFile.FileName + demoString;
  tempStream.WriteBuffer(tempString[1], length(tempString));
  {If the frame is not new (not empty) you should use the clearFrameData function
  here before setting the new data. Even though the setFrameData overwrites
  previous data, if the new data is smaller than the old, some of the old data
  will still exist at the end.}
  myTag.setFrameData(myTag.Frames.Count, tempStream, 0, false);
  tempStream.free;
  tempStream := TFileStream.create(openFile.filename, fmOpenRead or fmShareDenyNone);
  {This line writes the file into the frame after the data that was already
   written to it by using the 'offset' variable.}
  myTag.setFrameData(myTag.Frames.Count, tempStream, myTag.Frames.getSize(myTag.frames.count), false);
  tempStream.free;
  FramesListBox.items.add(format('%d.) ''%s''', [myTag.Frames.Count, 'GEOB']));

  end;
 end;
end;
{end***************************************************************************}

procedure TForm1.openTagButtonClick(Sender: TObject);
begin
if openTag.execute then
 begin
 if fileExists(openTag.FileName) then
  id3v2tagfile.text := openTag.fileName;
 end;
end;

initialization
myTag := Tid3v2tag.create;

finalization
myTag.free;

end.
